@@ -11,6 +11,7 @@ import (
 	"io"
 	"net/http"
 	"net/url"
+	"path/filepath"
 	"strconv"
 	"strings"
 	"time"
@@ -448,6 +449,10 @@ func parseNamespace(params map[string]string) (string, error) {
 		return "", err
 	}
 
+	if !filepath.IsLocal(namespace) {
+		return "", errors.New("invalid namespace: path traversal not allowed")
+	}
+
 	return namespace, nil
 }
 
@@ -464,6 +469,10 @@ func parseGroupName(params map[string]string) (string, error) {
 		return "", err
 	}
 
+	if !filepath.IsLocal(groupName) {
+		return "", errors.New("invalid groupname: path traversal not allowed")
+	}
+
 	return groupName, nil
 }
 
