@@ -5,17 +5,6 @@
 
 package util
 
-// StringsContain returns true if the search value is within the list of input values.
-func StringsContain(values []string, search string) bool {
-	for _, v := range values {
-		if search == v {
-			return true
-		}
-	}
-
-	return false
-}
-
 // StringsMap returns a map where keys are input values.
 func StringsMap(values []string) map[string]bool {
 	out := make(map[string]bool, len(values))
